@@ -1,7 +1,7 @@
 package Sidef {
 
     use 5.014;
-    our $VERSION = '2.11';
+    our $VERSION = '2.12';
 
     our $SPACES      = 0;    # the current number of spaces
     our $SPACES_INCR = 4;    # the number of spaces incrementor
@@ -18,6 +18,26 @@ package Sidef {
     sub new {
         bless {}, __PACKAGE__;
     }
+
+    sub normalize_type {
+        my ($type) = @_;
+
+        if (index($type, 'Sidef::') == 0) {
+            $type = substr($type, rindex($type, '::') + 2);
+        }
+        else {
+            $type =~ s/^(?:_::)?main:://
+              or $type =~ s/^_:://;
+        }
+
+        $type;
+    }
+
+    sub normalize_method {
+        my ($type, $method) = ($_[0] =~ /^(.*[^:])::(.*)$/);
+        normalize_type($type) . ".$method";
+    }
+
 };
 
 #
@@ -36,7 +56,7 @@ package Sidef {
     $self = ref($self) if ref($self);
 
     index($self, 'Sidef::') == 0
-      or die("[AUTOLOAD] Undefined method: $AUTOLOAD");
+      or die("[AUTOLOAD] Undefined method `" . Sidef::normalize_method($AUTOLOAD) . q{'});
 
     eval { require $self =~ s{::}{/}rg . '.pm' };
 
@@ -49,12 +69,11 @@ package Sidef {
         }
     }
 
-    my $func = \&{$AUTOLOAD};
-    if (defined(&$func)) {
-        return $func->($self, @args);
+    if (defined(&$AUTOLOAD)) {
+        return $AUTOLOAD->($self, @args);
     }
 
-    die "[AUTOLOAD] Undefined method: $AUTOLOAD";
+    die("[AUTOLOAD] Undefined method `" . Sidef::normalize_method($AUTOLOAD) . q{'});
     return;
 };
 
