@@ -39,12 +39,12 @@ package Sidef::Deparse::Perl {
                 '\\\\=' => '//=',
                         },
 
-            overload_methods => {
-                                 to_str  => q{""},
-                                 to_s    => q{""},
-                                 to_bool => q{bool},
-                                 to_num  => q{0+},
-                                },
+            #~ overload_methods => {
+            #~ to_str  => q{""},
+            #~ to_s    => q{""},
+            #~ to_bool => q{bool},
+            #~ to_num  => q{0+},
+            #~ },
 
             special_constructs => {
                                    'Sidef::Types::Block::If'    => 1,
@@ -111,10 +111,10 @@ HEADER
 
         my $ref = ref($obj);
         $self->_dump_string(
-                              $ref eq 'Sidef::Variable::ClassInit'    ? $self->_dump_class_name($obj)
-                            : $ref eq 'Sidef::Variable::Ref'          ? 'REF'
-                            : $ref eq 'Sidef::Types::Block::CodeInit' ? 'Sidef::Types::Block::Code'
-                            :                                           $ref
+                              $ref eq 'Sidef::Variable::ClassInit'     ? $self->_dump_class_name($obj)
+                            : $ref eq 'Sidef::Variable::Ref'           ? 'REF'
+                            : $ref eq 'Sidef::Types::Block::BlockInit' ? 'Sidef::Types::Block::Block'
+                            :                                            $ref
                            );
     }
 
@@ -138,7 +138,9 @@ HEADER
     sub _dump_var {
         my ($self, $var, $refaddr) = @_;
 
-        $var->{in_use} || exists($var->{value}) || exists($var->{ref_type}) || return 'undef';
+        $var->{name} // return 'undef';
+
+        # $var->{in_use} || exists($var->{value}) || exists($var->{ref_type}) || return 'undef';
 
         (
            exists($var->{array}) ? '@'
@@ -190,7 +192,8 @@ HEADER
                 }
 
                 push @code, (' ' x $Sidef::SPACES) . "\$$name = Sidef::Types::Array::Array->new(\@$name);\n";
-                $var->{in_use} ||= 1;
+
+                #$var->{in_use} ||= 1;
                 delete $var->{array};
             }
             elsif (exists $var->{hash}) {
@@ -207,7 +210,8 @@ HEADER
                 }
 
                 push @code, (' ' x $Sidef::SPACES) . "\$$name = Sidef::Types::Hash::Hash->new(\%$name);\n";
-                $var->{in_use} ||= 1;
+
+                #$var->{in_use} ||= 1;
                 delete $var->{hash};
             }
             elsif (exists $var->{value}) {
@@ -228,72 +232,51 @@ HEADER
         }
 
         # Return one var as a list
-        '(' . join(',', @code) . ')';
+        '(' . join(', ', @code) . ')';
+    }
+
+    sub _dump_class_attributes {
+        my ($self, @attrs) = @_;
+
+        my @code;
+        foreach my $attr (@attrs) {
+
+            my @vars = @{$attr->{vars}};
+            @vars || next;
+
+            my @dumped_vars = map { ref($_) ? $self->_dump_var($_) : $_ } @vars;
+
+            push @code,
+              (   'my('
+                . join(', ', @dumped_vars) . ')'
+                . (exists($attr->{args}) ? '=' . $self->deparse_args($attr->{args}) : ''));
+            foreach my $var (@vars) {
+                if (exists $var->{value}) {
+                    my $value = $self->deparse_expr({self => $var->{value}});
+                    if ($value ne '') {
+                        push @code, "\$$var->{name}" . refaddr($var) . " //= " . $value . ";";
+                    }
+                }
+            }
+
+        }
+
+        @code ? ((' ' x $Sidef::SPACES) . join(";\n" . (' ' x $Sidef::SPACES), @code) . ";\n") : '';
     }
 
     sub _dump_sub_init_vars {
-        my ($self, %opt) = @_;
+        my ($self, @vars) = @_;
 
-        my @vars = @{$opt{vars}};
         @vars || return '';
 
         my @dumped_vars = map { ref($_) ? $self->_dump_var($_) : $_ } @vars;
 
-        # Return when all variables are "undef" (i.e.: not in use)
+        # Return when all variables are "undef"
         if (all { $_ eq 'undef' } @dumped_vars) {
             return '';
         }
 
-        my $code;
-
-        if ($opt{extended}) {
-            $code = (' ' x $Sidef::SPACES) . join(
-                "\n" . (' ' x $Sidef::SPACES),
-                join(
-                    '',
-                    map { s/^\s+//r }
-                      "my \@__vars__ = (" . join(', ', map { $_ eq 'undef' ? $_ : "\\my $_" } @dumped_vars) . ');',
-                    "{state \$table = {"
-                      . join(', ', map { ref($vars[$_]) ? "$vars[$_]{name} => $_" : () } 0 .. $#vars) . "};",
-                    'my (@left, %seen);',
-                    q(foreach my $arg(@_) {),
-                    q(    if (ref($arg) eq 'Sidef::Variable::NamedParam') {),
-                    q(        if (exists $table->{$arg->[0]}) {),
-                    q(            my $var = $__vars__[$table->{$arg->[0]}] // next;),
-                    q(            if (ref($var) eq 'SCALAR') { ),
-                    q(               $$var = ${$arg->[1]}[-1];),
-                    q(            } elsif (ref($var) eq 'ARRAY') {),
-                    q(              @$var = @{$arg->[1]};),
-                    q(            } else {),
-                    q(              %$var = @{$arg->[1]};),
-                    q(            }),
-                    q(            undef $seen{$var};),
-                    q(         } else {),
-                    q(             die "No such named parameter: <<$arg->[0]>>";),
-                    q(         }),
-                    q(     } else {),
-                    q(        push @left, $arg;),
-                    q(     }),
-                    q(}),
-
-                    q(foreach my $var(@__vars__) {),
-                    q(    next if exists $seen{$var // do{shift @left; next}};),
-                    q(    @left || last;),
-                    q(    if (ref($var) eq 'SCALAR') {),
-                    q(      $$var = shift @left;),
-                    q(    } elsif (ref($var) eq 'ARRAY') {),
-                    q(      @$var = @left; last;),
-                    q(    } else {),
-                    q(      %$var = @left; last;),
-                    q(    }),
-                    q(}}),
-                    )
-              )
-              . "\n";
-        }
-        else {
-            $code = (' ' x $Sidef::SPACES) . "my (" . join(', ', @dumped_vars) . ') = @_;' . "\n";
-        }
+        my $code = (' ' x $Sidef::SPACES) . "my (" . join(', ', @dumped_vars) . ') = @_;' . "\n";
 
         foreach my $var (@vars) {
 
@@ -328,16 +311,6 @@ HEADER
                         $code .= (' ' x $Sidef::SPACES) . "\$$var->{name}" . refaddr($var) . " //= " . $value . ";\n";
                     }
                 }
-
-                if (exists($var->{ref_type})) {
-                    my $var_name = "\$$var->{name}" . refaddr($var);
-                    my $type     = $self->_dump_reftype($var->{ref_type});
-                    $code .=
-                        (' ' x $Sidef::SPACES)
-                      . "(ref($var_name) eq $type || ($type ne 'REF' && eval{$var_name->SUPER::isa($type)})) or "
-                      . "die q{[ERROR] Invalid type-parameter for variable <<$var->{name}>> in $self->{parent_name}[0] <<$self->{parent_name}[1]>>: got <<} . "
-                      . "ref($var_name) . q{>>, but expected $type};\n";
-                }
             }
         }
 
@@ -393,6 +366,36 @@ HEADER
           . '}';
     }
 
+    sub _dump_var_attr {
+        my ($self, @vars) = @_;
+
+        'vars => [' . join(
+            ', ',
+            map {
+                    "{ name => "
+                  . $self->_dump_string($_->{name})
+                  . (exists($_->{slurpy})    ? (", slurpy => " . $_->{slurpy})                       : '')
+                  . (exists($_->{ref_type})  ? (", type => " . $self->_dump_reftype($_->{ref_type})) : '')
+                  . (exists($_->{has_value}) ? (', has_value => 1')                                  : '')
+                  . (
+                     exists($_->{where_block})
+                     ? (', where_block => sub{'
+                        . $self->_dump_sub_init_vars($_->{where_block}{init_vars}{vars}[0])
+                        . $self->deparse_generic('', ';', '', $_->{where_block}{code}) . '}')
+                     : ''
+                    )
+                  . (
+                     exists($_->{where_expr}) ? (', where_expr => do{' . $self->deparse_expr({self => $_->{where_expr}}) . '}')
+                     : ''
+                    )
+                  . '}'
+              } @vars
+          )
+          . ']' . ', '
+          . 'table => {'
+          . join(', ', map { $self->_dump_string($vars[$_]{name}) . ' => ' . $_ } 0 .. $#vars) . '}';
+    }
+
     sub _dump_class_name {
         my ($self, $class) = @_;
         join('::', '_', $class->{class}, $class->{name});
@@ -489,25 +492,26 @@ HEADER
                     # Deparse the block of the method/function
                     {
                         local $self->{function} = refaddr($block);
-                        local $self->{parent_name} = [$obj->{type}, $name];
+                        local $self->{parent_name} = [$obj->{type}, "$obj->{class}::$name"];
                         push @{$self->{function_declarations}}, [$self->{function}, "my \$$obj->{name}$refaddr;"];
                         $code .= $self->deparse_expr({self => $block});
                     }
 
                     # Check to see if the method/function has kids (can do multiple dispatch)
                     if (exists $obj->{value}{kids}) {
-                        die "[ERROR] Multiple dispatch is currently unsupported!";
+                        chop $code;
 
-                        # my $deparsed_block = $self->deparse_expr({self => $block});
-                        #my @kids = map{$self->deparse_expr({self=>$_})}@{$obj->{value}{kids}};
-                        #die join('', @kids);
-                        #$code .= 'Sidef::Types::Block::MultiDispatch->new(' . join(', ',  $deparsed_block, @kids). ')';
+                        my @kids = map {
+                            local $_->{type} = 'func';
+                            "do{" . $self->deparse_expr({self => $_}) . "}";
+                        } @{$obj->{value}{kids}};
 
+                        $code .= ', kids => [' . join(', ', @kids) . '])';
                     }
 
                     # Check the return value (when "-> Type" is specified)
                     if (exists $obj->{returns}) {
-                        my $types = '[' . join(',', map { $self->_dump_reftype($_) } @{$obj->{returns}}) . ']';
+                        my $types = '[' . join(', ', map { $self->_dump_reftype($_) } @{$obj->{returns}}) . ']';
                         $code = "do {$code; \$$obj->{name}$refaddr\->{returns} = $types; \$$obj->{name}$refaddr}";
                     }
 
@@ -537,11 +541,6 @@ HEADER
 
                         # Other methods
                         else {
-
-                            ## Old way
-                          # $code .= ";\n" . (' ' x $Sidef::SPACES) . "sub $obj->{name} { \$$obj->{name}$refaddr->call(\@_) }";
-
-                            # New way
                             $code .= ";\n"
                               . (' ' x $Sidef::SPACES)
                               . "state \$_$refaddr = do { no strict 'refs'; *{"
@@ -550,12 +549,12 @@ HEADER
                         }
 
                         # Add the "overload" pragma for some special methods
-                        if (exists $self->{overload_methods}{$obj->{name}}) {
-                            $code .= ";\n"
-                              . (' ' x $Sidef::SPACES)
-                              . qq{use overload q{$self->{overload_methods}{$obj->{name}}} => }
-                              . $self->_dump_string("$self->{package_name}::$obj->{name}");
-                        }
+                        #~ if (exists $self->{overload_methods}{$obj->{name}}) {
+                        #~ $code .= ";\n"
+                        #~ . (' ' x $Sidef::SPACES)
+                        #~ . qq{use overload q{$self->{overload_methods}{$obj->{name}}} => }
+                        #~ . $self->_dump_string("$self->{package_name}::$obj->{name}");
+                        #~ }
                     }
                 }
             }
@@ -645,21 +644,27 @@ HEADER
                     $code .= ($package_name = $self->_dump_class_name($obj));
                 }
 
-                my $vars = $obj->{vars};
-                local $self->{class}        = refaddr($block);
-                local $self->{class_name}   = $obj->{name};
-                local $self->{parent_name}  = ['class initialization', $obj->{name}];
-                local $self->{package_name} = $package_name;
-                local $self->{inherit}      = $obj->{inherit} if exists $obj->{inherit};
-                local $self->{class_vars}   = $vars;
-                local $self->{ref_class}    = 1 if ref($obj->{name});
+                local $self->{class}            = refaddr($block);
+                local $self->{class_name}       = $package_name;
+                local $self->{parent_name}      = ['class', $package_name];
+                local $self->{package_name}     = $package_name;
+                local $self->{inherit}          = $obj->{inherit} if exists $obj->{inherit};
+                local $self->{class_vars}       = $obj->{vars} if exists $obj->{vars};
+                local $self->{class_attributes} = $obj->{attributes} if exists $obj->{attributes};
+                local $self->{ref_class}        = 1 if ref($obj->{name});
                 $code .= $self->deparse_expr({self => $block});
                 $code .= '; ' . $self->_dump_string($package_name) . '}';
             }
         }
-        elsif ($ref eq 'Sidef::Types::Block::CodeInit') {
+        elsif ($ref eq 'Sidef::Types::Block::BlockInit') {
             if ($addr{$refaddr}++) {
-                $code = 'Sidef::Types::Block::Code->new(code => __SUB__)';
+                $code = 'Sidef::Types::Block::Block->new(code => __SUB__';
+
+                if (exists($obj->{init_vars}) and @{$obj->{init_vars}{vars}}) {
+                    $code .= ', ' . $self->_dump_var_attr(@{$obj->{init_vars}{vars}});
+                }
+
+                $code .= ')';
             }
             else {
                 if (%{$obj}) {
@@ -686,35 +691,51 @@ HEADER
                               . ($self->{package_name} eq 'Sidef::Object::Object' ? '' : "Sidef::Object::Object") . ");\n";
                         }
 
-                        if ($is_class and exists $self->{class_vars} and not $self->{ref_class}) {
+                        if ($is_class and not $self->{ref_class}) {
 
-                            $code .= (" " x $Sidef::SPACES) . 'sub new {' . "\n";
+                            $code .=
+                              (" " x $Sidef::SPACES) . "\$new$refaddr = Sidef::Types::Block::Block->new(code => sub {" . "\n";
+                            push @{$self->{function_declarations}}, [$refaddr, "my \$new$refaddr;"];
 
                             $Sidef::SPACES += $Sidef::SPACES_INCR;
-                            $code .= $self->_dump_sub_init_vars(extended => 1, vars => ['undef', @{$self->{class_vars}}]);
+                            $code .= $self->_dump_sub_init_vars(@{$self->{class_vars}})
+                              . $self->_dump_class_attributes(@{$self->{class_attributes}});
 
-                            $code .= " " x $Sidef::SPACES;
-                            $code .= 'my $self = bless {';
-                            foreach my $var (@{$self->{class_vars}}) {
-                                $code .= qq{"\Q$var->{name}\E"=>} . $self->_dump_var($var) . ',';
+                            $code .= (' ' x $Sidef::SPACES) . 'my $self = bless {';
+                            foreach my $var (@{$self->{class_vars}}, map { @{$_->{vars}} } @{$self->{class_attributes}}) {
+                                $code .= qq{"\Q$var->{name}\E"=>} . $self->_dump_var($var) . ', ';
                             }
 
-                            $code .= '}, __PACKAGE__;' . "\n";
-                            $code .= (" " x $Sidef::SPACES) . '$self->init() if $self->can("init");' . "\n";
-                            $code .= (" " x $Sidef::SPACES) . '$self;' . "\n";
+                            $code .=
+                                '}, __PACKAGE__;' . "\n"
+                              . (' ' x $Sidef::SPACES)
+                              . '$self->init() if $self->can("init");' . "\n"
+                              . (' ' x $Sidef::SPACES)
+                              . '$self;' . "\n";
 
                             $Sidef::SPACES -= $Sidef::SPACES_INCR;
-                            $code .= " " x $Sidef::SPACES . "}";
-                            $code .= "\n" . (' ' x $Sidef::SPACES) . "*call = \\&new;\n";
+                            $code .=
+                                (" " x $Sidef::SPACES . "}") . ', '
+                              . $self->_dump_var_attr(@{$self->{class_vars}})
+                              . ", type => "
+                              . $self->_dump_string('class')
+                              . ", name => "
+                              . $self->_dump_string($self->{parent_name}[1]) . ");\n"
+                              . (' ' x $Sidef::SPACES)
+                              . "state \$_$refaddr = do { no strict 'refs'; *{"
+                              . $self->_dump_string("$self->{package_name}\::new")
+                              . "} = *{"
+                              . $self->_dump_string("$self->{package_name}\::call")
+                              . "} = sub { CORE::shift(\@_); \$new$refaddr->call(\@_) } };\n";
 
-                            foreach my $var (@{$self->{class_vars}}) {
-                                $code .= " " x $Sidef::SPACES;
-                                $code .= qq{sub $var->{name} : lvalue { \$_[0]->{"\Q$var->{name}\E"} }\n};
+                            foreach my $var (@{$self->{class_vars}}, map { @{$_->{vars}} } @{$self->{class_attributes}}) {
+                                $code .=
+                                  (' ' x $Sidef::SPACES) . qq{sub $var->{name} : lvalue { \$_[0]->{"\Q$var->{name}\E"} }\n};
                             }
                         }
                     }
                     else {
-                        $code = 'Sidef::Types::Block::Code->new(';
+                        $code = 'Sidef::Types::Block::Block->new(';
                     }
 
                     if (not $is_class) {
@@ -722,18 +743,11 @@ HEADER
                         $code .= "\n" . (" " x ($Sidef::SPACES - $Sidef::SPACES_INCR)) . "code => sub {\n";
 
                         if (exists($obj->{init_vars}) and @{$obj->{init_vars}{vars}}) {
-                            my @vars = @{$obj->{init_vars}{vars}};
+                            $code .= $self->_dump_sub_init_vars(@{$obj->{init_vars}{vars}});
+                        }
 
-                            # Remove the underscore (_) variable
-                            if (@vars > 1) {
-                                pop @vars;
-                            }
-
-                            $code .= $self->_dump_sub_init_vars(extended => $is_function, vars => \@vars);
-
-                            if ($is_function) {
-                                $code .= (' ' x $Sidef::SPACES) . 'my @return;' . "\n";
-                            }
+                        if ($is_function) {
+                            $code .= (' ' x $Sidef::SPACES) . 'my @return;' . "\n";
                         }
                     }
 
@@ -776,59 +790,73 @@ HEADER
                       . (" " x ($Sidef::SPACES -= $Sidef::SPACES_INCR)) . '}';
 
                     if (not $is_class) {
-                        if (exists $self->{parent_name}) {
-                            $code .= ','
-                              . join(',',
+                        if ($is_function) {
+                            $code .= ', '
+                              . join(', ',
                                      'type => ' . $self->_dump_string($self->{parent_name}[0]),
                                      'name => ' . $self->_dump_string($self->{parent_name}[1]));
                         }
+                        else {
+                            $code .= ', '
+                              . join(', ',
+                                     'type => ' . $self->_dump_string('block'),
+                                     'name => ' . $self->_dump_string('__BLOCK__'),);
+                        }
 
                         if (exists $self->{class_name}) {
-                            $code .= ',' . 'class => ' . $self->_dump_string($self->{class_name});
+                            $code .= ', ' . 'class => ' . $self->_dump_string($self->{class_name});
+                        }
+
+                        if (exists($obj->{init_vars}) and @{$obj->{init_vars}{vars}}) {
+                            $code .= ', ' . $self->_dump_var_attr(@{$obj->{init_vars}{vars}});
                         }
                         $code .= ')';
                     }
                 }
                 else {
-                    $code = 'Block';
+                    $code = q{'Sidef::Types::Block::Block'};
                 }
             }
         }
+        elsif ($ref eq 'Sidef::Variable::ClassAttr') {
+            ## ok
+        }
         elsif ($ref eq 'Sidef::Variable::Struct') {
             my $name = $self->_dump_class_name($obj);
             if ($addr{$refaddr}++) {
                 $code = $name;
             }
             else {
-                local $self->{parent_name} = ['struct initialization', $obj->{name}];
-
-                # Mark the variables all in use
-                foreach my $var (@{$obj->{vars}}) {
-                    $var->{in_use} = 1;
-                }
-
                 $Sidef::SPACES += $Sidef::SPACES_INCR;
                 $code =
                     "package $name {\n"
                   . (' ' x $Sidef::SPACES)
-                  . "sub new {\n"
+                  . "\$new$refaddr = Sidef::Types::Block::Block->new(code => sub {" . "\n"
                   . (' ' x $Sidef::SPACES)
-                  . $self->_dump_sub_init_vars(extended => 1, vars => ['undef', @{$obj->{vars}}])
-
+                  . $self->_dump_sub_init_vars(@{$obj->{vars}})
                   . (' ' x ($Sidef::SPACES * 2))
                   . "bless {"
                   . join(", ", map { $self->_dump_string($_->{name}) . " => " . $self->_dump_var($_) } @{$obj->{vars}})
                   . "}, __PACKAGE__" . "\n"
-                  . (' ' x $Sidef::SPACES) . "}\n"
-                  .
-
-                  (' ' x $Sidef::SPACES) . "*call = \\&new;\n" .
-
-                  (' ' x $Sidef::SPACES)
+                  . (' ' x $Sidef::SPACES) . "}" . ', '
+                  . 'name => '
+                  . $self->_dump_string($name) . ', '
+                  . 'type => '
+                  . $self->_dump_string('struct') . ', '
+                  . $self->_dump_var_attr(@{$obj->{vars}}) . ");\n" . ";\n"
+                  . (' ' x $Sidef::SPACES)
+                  . "state \$_$refaddr = do { no strict 'refs'; *{"
+                  . $self->_dump_string("$name\::new")
+                  . "} = *{"
+                  . $self->_dump_string("$name\::call")
+                  . "} = sub { CORE::shift(\@_); \$new$refaddr->call(\@_) } };\n"
+                  . (' ' x $Sidef::SPACES)
                   . join("\n" . (' ' x $Sidef::SPACES),
                          map { "sub $_->{name} : lvalue { \$_[0]->{$_->{name}} }" } @{$obj->{vars}})
+                  . "\n"
+                  . (' ' x ($Sidef::SPACES - $Sidef::SPACES_INCR)) . "}";
 
-                  . "\n" . (' ' x ($Sidef::SPACES - $Sidef::SPACES_INCR)) . "}";
+                push @{$self->{function_declarations}}, [$refaddr, "my \$new$refaddr;"];
 
                 $Sidef::SPACES -= $Sidef::SPACES_INCR;
             }
@@ -872,7 +900,7 @@ HEADER
               . $self->deparse_block_expr($obj->{false}) . ')';
         }
         elsif ($ref eq 'Sidef::Variable::NamedParam') {
-            $code = $ref . '->new(' . $self->_dump_string($obj->[0]) . ',' . $self->deparse_args(@{$obj->[1]}) . ')';
+            $code = $ref . '->new(' . $self->_dump_string($obj->[0]) . ', ' . $self->deparse_args(@{$obj->[1]}) . ')';
         }
         elsif ($ref eq 'Sidef::Types::Nil::Nil') {
             $code = 'undef';
@@ -880,7 +908,7 @@ HEADER
         elsif ($ref eq 'Sidef::Types::Hash::Hash') {
             if (keys(%{$obj})) {
                 $code = $ref . '->new(' . join(
-                    ',',
+                    ', ',
                     map {
                         $self->_dump_string($_) . ' => '
                           . (defined($obj->{$_}) ? $self->deparse_expr({self => $obj->{$_}}) : 'undef')
@@ -893,16 +921,25 @@ HEADER
             }
         }
         elsif ($ref eq 'Sidef::Types::Block::Do') {
-            $code = 'do {' . join(';', $self->deparse_script($obj->{block}{code})) . '}';
+            $code = 'do ' . $self->deparse_bare_block($obj->{block}{code});
         }
         elsif ($ref eq 'Sidef::Types::Block::Loop') {
-            $code = 'while(1) {' . join(';', $self->deparse_script($obj->{block}{code})) . '}';
+            $code = 'while(1) ' . $self->deparse_bare_block($obj->{block}{code});
         }
         elsif ($ref eq 'Sidef::Types::Block::Given') {
             $self->top_add(qq{use experimental 'smartmatch';\n});
+            my $dvar = $self->_dump_var($obj->{block}{init_vars}->{vars}[0]);
+            $code =
+                'do{given (my '
+              . $dvar . '='
+              . $self->deparse_args($obj->{expr}) . ')'
+              . $self->deparse_bare_block($obj->{block}{code}) . '}';
         }
         elsif ($ref eq 'Sidef::Types::Block::When') {
-            $self->top_add(qq{use experimental 'smartmatch';\n});
+            $code = 'when($_ ~~ ' . $self->deparse_args($obj->{expr}) . ')' . $self->deparse_bare_block($obj->{block}{code});
+        }
+        elsif ($ref eq 'Sidef::Types::Block::Case') {
+            $code = 'when(!!' . $self->deparse_args($obj->{expr}) . ')' . $self->deparse_bare_block($obj->{block}{code});
         }
         elsif ($ref eq 'Sidef::Types::Block::Default') {
             $code = 'default' . $self->deparse_bare_block($obj->{block}->{code});
@@ -917,12 +954,15 @@ HEADER
             my $raddr = refaddr($obj->{gather});
             $code = "do { push \@_$raddr," . $self->deparse_args($obj->{expr}) . "; \$_$raddr\[-1] }";
         }
-        elsif ($ref eq 'Sidef::Types::Block::For') {
-            ## ok
-        }
         elsif ($ref eq 'Sidef::Types::Block::Try') {
             $code = $ref . '->new';
         }
+        elsif ($ref eq 'Sidef::Variable::Ref') {
+            ## ok
+        }
+        elsif ($ref eq 'Sidef::Types::Block::For') {
+            ## ok
+        }
         elsif ($ref eq 'Sidef::Types::Block::Break') {
             $code = 'last';
         }
@@ -973,9 +1013,6 @@ HEADER
         elsif ($ref eq 'Sidef::Types::Glob::Socket') {
             $code = $self->make_constant($ref, 'new', "Socket$refaddr");
         }
-        elsif ($ref eq 'Sidef::Perl::Perl') {
-            $code = $self->make_constant($ref, 'new', "Perl$refaddr");
-        }
         elsif ($ref eq 'Sidef::Eval::Eval') {
             $Sidef::EVALS{$refaddr} = $obj;
             $code = qq~
@@ -987,6 +1024,8 @@ HEADER
             local \$Sidef::DEPARSER->{block_declarations} = [];
             \$Sidef::DEPARSER->deparse(
             do {
+                local \$Sidef::PARSER->{line} = 0;
+                local \$Sidef::PARSER->{file_name} = 'eval($refaddr)';
                 local \$Sidef::PARSER->{vars} = \$Sidef::EVALS{$refaddr}{vars};
                 local \$Sidef::PARSER->{ref_vars_refs} = \$Sidef::EVALS{$refaddr}{ref_vars_refs};
                 \$Sidef::PARSER->parse_script(code => \\(~ . $self->deparse_script($obj->{expr}) . qq~->get_value));
@@ -1047,8 +1086,8 @@ HEADER
                   . " $obj->{file} line $obj->{line} (expected 1 argument)\n";
 
                 # Generate code
-                $code = qq~do{do{$args[0]} or CORE::die "$obj->{act}\Q$obj->{code}\E failed ~
-                  . qq~in \Q$obj->{file}\E at line $obj->{line}\\n"}~;
+                $code = qq~do{my \$a$refaddr = do{$args[0]}; \$a$refaddr or CORE::die "$obj->{act}(\$a$refaddr) failed ~
+                  . qq~at \Q$obj->{file}\E line $obj->{line}\\n"}~;
             }
             elsif ($obj->{act} eq 'assert_eq' or $obj->{act} eq 'assert_ne') {
 
@@ -1059,8 +1098,10 @@ HEADER
 
                 # Generate code
                 $code = "do{"
-                  . ($obj->{act} eq 'assert_ne' ? qq{not(do{$args[0]} eq do{$args[1]})} : qq{do{$args[0]} eq do{$args[1]}})
-                  . qq{ or CORE::die "$obj->{act}\Q$obj->{code}\E failed at \Q$obj->{file}\E line $obj->{line}\\n\"\}};
+                  . "my \$a$refaddr = do{$args[0]};"
+                  . "my \$b$refaddr = do{$args[1]};"
+                  . ($obj->{act} eq 'assert_ne' ? qq{CORE::not(\$a$refaddr eq \$b$refaddr)} : qq{\$a$refaddr eq \$b$refaddr})
+                  . qq~ or CORE::die "$obj->{act}(\$a$refaddr, \$b$refaddr) failed at \Q$obj->{file}\E line $obj->{line}\\n"}~;
             }
         }
         elsif ($ref eq 'Sidef::Meta::Error') {
@@ -1103,17 +1144,23 @@ HEADER
             $code = $self->make_constant($ref, 'new', "Grapheme$refaddr", $self->_dump_string(${$obj}));
         }
         elsif ($ref eq 'Sidef::Types::Array::MultiArray') {
-            $code = $ref . '->new';
+            $code = $self->make_constant($ref, 'new', "MultiArr$refaddr");
         }
         elsif ($ref eq 'Sidef::Types::Glob::Pipe') {
             $code = $self->make_constant($ref, 'new', "Pipe$refaddr", map { $self->_dump_string($_) } @{$obj});
         }
         elsif ($ref eq 'Sidef::Parser') {
-            $code = $ref . '->new';
+            $code = $self->make_constant($ref, 'new', "Parser$refaddr");
         }
         elsif ($ref eq 'Sidef') {
             $code = $self->make_constant($ref, 'new', "Sidef$refaddr");
         }
+        elsif ($ref eq 'Sidef::Perl::Perl') {
+            $code = $self->make_constant($ref, 'new', "Perl$refaddr");
+        }
+        elsif ($ref eq 'Sidef::Meta::Unimplemented') {
+            $code = qq{CORE::die "Unimplemented at " . } . $self->_dump_string($obj->{file}) . qq{. " line $obj->{line}\\n"};
+        }
 
         # Array indices
         if (exists $expr->{ind}) {
@@ -1220,14 +1267,19 @@ HEADER
                         # Prefix ++ and -- operators on variables
                         elsif (exists $self->{inc_dec_ops}{$method}) {
                             my $var = $self->deparse_args(@{$call->{arg}});
-                            $code = "($var=$var\->$self->{inc_dec_ops}{$method})[0]";
+
+                            #$code = "($var=$var\->$self->{inc_dec_ops}{$method})[0]";
+                            $code = "do{my \$ref=\\$var; \$\$ref=\$\$ref\->$self->{inc_dec_ops}{$method}}";
                             next;
                         }
                     }
 
                     # Postfix ++ and -- operators on variables
                     if (exists($self->{inc_dec_ops}{$method})) {
-                        $code = "do{my \$old=$code; $code=$code\->$self->{inc_dec_ops}{$method}; \$old}";
+
+                        #$code = "do{my \$old=$code; $code=$code\->$self->{inc_dec_ops}{$method}; \$old}";
+                        $code =
+                          "do{my \$ref=\\$code; my \$value=\$\$ref; \$\$ref=\$value\->$self->{inc_dec_ops}{$method}; \$value}";
                         next;
                     }
 
@@ -1253,8 +1305,11 @@ HEADER
 
                     # != and == methods
                     if ($method eq '==' or $method eq '!=') {
-                        $code = 'Sidef::Types::Bool::Bool->new(' . $code . 'eq' . $self->deparse_args(@{$call->{arg}}) . ')';
-                        $code .= '->not' if ($method eq '!=');
+                        $code =
+                            'Sidef::Types::Bool::Bool->new('
+                          . ($method eq '!=' ? 'CORE::not' : '') . '('
+                          . $code . 'eq'
+                          . $self->deparse_args(@{$call->{arg}}) . '))';
                         next;
                     }
 
@@ -1300,13 +1355,13 @@ HEADER
                             next;
                         }
 
-                        if ($method eq '>' or $method eq 'say') {
-                            $code = 'Sidef::Types::Bool::Bool->new(CORE::say ' . $self->deparse_args(@{$call->{arg}}) . ')';
+                        if ($method eq 'say' or $method eq '>') {
+                            $code = 'Sidef::Types::Bool::Bool->new(CORE::say' . $self->deparse_args(@{$call->{arg}}) . ')';
                             next;
                         }
 
-                        if ($method eq '>>' or $method eq 'print') {
-                            $code = 'Sidef::Types::Bool::Bool->new(CORE::print ' . $self->deparse_args(@{$call->{arg}}) . ')';
+                        if ($method eq 'print' or $method eq '>>') {
+                            $code = 'Sidef::Types::Bool::Bool->new(CORE::print' . $self->deparse_args(@{$call->{arg}}) . ')';
                             next;
                         }
                     }
@@ -1362,8 +1417,9 @@ HEADER
                 }
 
                 if (exists $call->{block}) {
-                    if ($ref eq 'Sidef::Types::Block::Given'
-                        or ($ref eq 'Sidef::Types::Block::If' and $i == $#{$expr->{call}})) {
+
+                    # TODO: move above the deparsing of `Block::If`
+                    if ($ref eq 'Sidef::Types::Block::If' and $i == $#{$expr->{call}}) {
                         $code = "do {\n" . (' ' x $Sidef::SPACES) . $code . $self->deparse_bare_block(@{$call->{block}}) . '}';
                     }
                     else {
