@@ -40,11 +40,6 @@ package Sidef::Deparse::Sidef {
         $self->deparse_generic('(', ', ', ')', @args);
     }
 
-    sub deparse_block_expr {
-        my ($self, @args) = @_;
-        $self->deparse_generic('do{', ';', '}', @args);
-    }
-
     sub deparse_bare_block {
         my ($self, @args) = @_;
 
@@ -59,8 +54,8 @@ package Sidef::Deparse::Sidef {
     }
 
     sub _dump_init_vars {
-        my ($self, $init_obj) = @_;
-        my $code = 'var(' . $self->_dump_vars(@{$init_obj->{vars}}) . ')';
+        my ($self, $init_obj, $type) = @_;
+        my $code = $type . '(' . $self->_dump_vars(@{$init_obj->{vars}}) . ')';
 
         if (exists $init_obj->{args}) {
             $code .= '=' . $self->deparse_args($init_obj->{args});
@@ -74,9 +69,9 @@ package Sidef::Deparse::Sidef {
 
         my $ref = ref($obj);
 
-            $ref eq 'Sidef::Variable::ClassInit'    ? $obj->{name}
-          : $ref eq 'Sidef::Types::Block::CodeInit' ? 'Block'
-          :                                           substr($ref, rindex($ref, '::') + 2);
+            $ref eq 'Sidef::Variable::ClassInit'     ? $obj->{name}
+          : $ref eq 'Sidef::Types::Block::BlockInit' ? 'Block'
+          :                                            substr($ref, rindex($ref, '::') + 2);
     }
 
     sub _dump_vars {
@@ -88,13 +83,9 @@ package Sidef::Deparse::Sidef {
                   . (exists($_->{class}) && $_->{class} ne $self->{class} ? $_->{class} . '::' : '')
                   . (exists($_->{ref_type}) ? ($self->_dump_reftype($_->{ref_type}) . ' ') : '')
                   . $_->{name}
-                  . (
-                    exists($_->{value}) ? do {
-                        my $value = $self->deparse_expr({self => $_->{value}});
-                        "=($value)";
-                      }
-                    : ''
-                    )
+                  . (exists($_->{where_block}) ? $self->deparse_expr({self => $_->{where_block}}) : '')
+                  . (exists($_->{where_expr}) ? ('(' . $self->deparse_expr({self => $_->{where_expr}}) . ')') : '')
+                  . (exists($_->{value}) ? ('=(' . $self->deparse_expr({self => $_->{value}}) . ')') : '')
               } @vars
             );
     }
@@ -147,7 +138,7 @@ package Sidef::Deparse::Sidef {
             }
         }
         elsif ($ref eq 'Sidef::Variable::Variable') {
-            if ($obj->{type} eq 'var' or $obj->{type} eq 'static' or $obj->{type} eq 'const' or $obj->{type} eq 'def') {
+            if ($obj->{type} eq 'var' or $obj->{type} eq 'static' or $obj->{type} eq 'const') {
                 $code =
                   $obj->{name} =~ /^[0-9]+\z/
                   ? ('$' . $obj->{name})
@@ -175,7 +166,7 @@ package Sidef::Deparse::Sidef {
                     my $var_obj = delete $block->{init_vars};
 
                     $code .= '('
-                      . $self->_dump_vars(@{$var_obj->{vars}}[($obj->{type} eq 'method' ? 1 : 0) .. $#{$var_obj->{vars}} - 1])
+                      . $self->_dump_vars(@{$var_obj->{vars}}[($obj->{type} eq 'method' ? 1 : 0) .. $#{$var_obj->{vars}}])
                       . ') ';
 
                     if (exists $obj->{cached}) {
@@ -196,6 +187,9 @@ package Sidef::Deparse::Sidef {
                 }
             }
         }
+        elsif ($ref eq 'Sidef::Variable::ClassAttr') {
+            $code = $self->_dump_init_vars($obj, 'has');
+        }
         elsif ($ref eq 'Sidef::Variable::Struct') {
             if ($addr{refaddr($obj)}++) {
                 $code = $obj->{name};
@@ -211,7 +205,7 @@ package Sidef::Deparse::Sidef {
             $code = 'global ' . $obj->{class} . '::' . $obj->{name},;
         }
         elsif ($ref eq 'Sidef::Variable::Init') {
-            $code = $self->_dump_init_vars($obj);
+            $code = $self->_dump_init_vars($obj, 'var');
         }
         elsif ($ref eq 'Sidef::Variable::ConstInit') {
             $code = join(";\n" . (" " x $Sidef::SPACES), map { $self->deparse_expr({self => $_}) } @{$obj->{vars}});
@@ -270,8 +264,7 @@ package Sidef::Deparse::Sidef {
 
                 local $self->{class} = $obj->{class};
                 $code .= "class " . $self->_dump_class_name($obj->{name});
-                my $vars = $obj->{vars};
-                $code .= '(' . $self->_dump_vars(@{$vars}) . ')';
+                $code .= '(' . $self->_dump_vars(@{$obj->{vars}}) . ')';
                 if (exists $obj->{inherit}) {
                     $code .= ' << ' . join(', ', map { $_->{name} } @{$obj->{inherit}}) . ' ';
                 }
@@ -283,7 +276,7 @@ package Sidef::Deparse::Sidef {
                 }
             }
         }
-        elsif ($ref eq 'Sidef::Types::Block::CodeInit') {
+        elsif ($ref eq 'Sidef::Types::Block::BlockInit') {
             if ($addr{refaddr($obj)}++) {
                 $code = keys(%{$obj}) ? '__BLOCK__' : 'Block';
             }
@@ -292,7 +285,6 @@ package Sidef::Deparse::Sidef {
                     $code = '{';
                     if (exists($obj->{init_vars}) and @{$obj->{init_vars}{vars}}) {
                         my @vars = @{$obj->{init_vars}{vars}};
-                        pop @vars;
                         if (@vars) {
                             $code .= '| ' . $self->_dump_vars(@vars) . ' |';
                         }
@@ -305,13 +297,18 @@ package Sidef::Deparse::Sidef {
                         $statements[-1] = '(' . $statements[-1] . ')';
                     }
 
-                    $code .=
-                      @statements
-                      ? ("\n"
-                         . (" " x $Sidef::SPACES)
-                         . join(";\n" . (" " x $Sidef::SPACES), @statements) . "\n"
-                         . (" " x ($Sidef::SPACES - $Sidef::SPACES_INCR)) . '}')
-                      : '}';
+                    if (@statements == 1 and length($statements[0]) <= 80) {
+                        $code .= " $statements[0] }";
+                    }
+                    else {
+                        $code .=
+                          @statements
+                          ? ("\n"
+                             . (" " x $Sidef::SPACES)
+                             . join(";\n" . (" " x $Sidef::SPACES), @statements) . "\n"
+                             . (" " x ($Sidef::SPACES - $Sidef::SPACES_INCR)) . '}')
+                          : '}';
+                    }
 
                     $Sidef::SPACES -= $Sidef::SPACES_INCR;
                 }
@@ -337,6 +334,9 @@ package Sidef::Deparse::Sidef {
         elsif ($ref eq 'Sidef::Meta::Warning') {
             $code = 'warn' . $self->deparse_args($obj->{arg});
         }
+        elsif ($ref eq 'Sidef::Meta::Unimplemented') {
+            $code = '...';
+        }
         elsif ($ref eq 'Sidef::Eval::Eval') {
             $code = 'eval' . $self->deparse_args($obj->{expr});
         }
@@ -358,6 +358,15 @@ package Sidef::Deparse::Sidef {
         elsif ($ref eq 'Sidef::Types::Block::Break') {
             $code = 'break';
         }
+        elsif ($ref eq 'Sidef::Types::Block::Given') {
+            $code = 'given ' . $self->deparse_args($obj->{expr}) . $self->deparse_bare_block($obj->{block}{code});
+        }
+        elsif ($ref eq 'Sidef::Types::Block::When') {
+            $code = 'when(' . $self->deparse_args($obj->{expr}) . ')' . $self->deparse_bare_block($obj->{block}{code});
+        }
+        elsif ($ref eq 'Sidef::Types::Block::Case') {
+            $code = 'case(' . $self->deparse_args($obj->{expr}) . ')' . $self->deparse_bare_block($obj->{block}{code});
+        }
         elsif ($ref eq 'Sidef::Types::Block::Default') {
             $code = 'default' . $self->deparse_bare_block($obj->{block}->{code});
         }
@@ -613,7 +622,7 @@ package Sidef::Deparse::Sidef {
                     }
                     elsif ($method =~ /^[\pL_]/) {
 
-                        if ($ref eq 'Sidef::Types::Block::CodeInit' and $method eq 'loop') {
+                        if ($ref eq 'Sidef::Types::Block::BlockInit' and $method eq 'loop') {
                             $code = "loop $code";
                         }
                         else {
@@ -661,13 +670,7 @@ package Sidef::Deparse::Sidef {
                 }
 
                 if (exists $call->{block}) {
-                    if ($ref eq 'Sidef::Types::Block::Given'
-                        or ($ref eq 'Sidef::Types::Block::If' and $i == $#{$expr->{call}})) {
-                        $code .= $self->deparse_bare_block(@{$call->{block}});
-                    }
-                    else {
-                        $code .= $self->deparse_bare_block(@{$call->{block}});
-                    }
+                    $code .= $self->deparse_bare_block(@{$call->{block}});
                     next;
                 }
             }
