@@ -5,17 +5,39 @@ package Sidef::Object::Object {
 
     use overload
       q{~~}   => \&{__PACKAGE__ . '::' . '~~'},
-      q{bool} => sub { $_[0] },
-      q{0+}   => sub { $_[0] },
-      q{""}   => sub { $_[0] },
-      q{cmp}  => sub {
+      q{bool} => sub {
+        if (defined(my $sub = $_[0]->can('to_b'))) {
+            $sub->($_[0]);
+        }
+        else {
+            $_[0];
+        }
+      },
+      q{0+} => sub {
+        if (defined(my $sub = $_[0]->can('to_n'))) {
+            $sub->($_[0]);
+        }
+        else {
+            $_[0];
+        }
+      },
+      q{""} => sub {
+        if (defined(my $sub = $_[0]->can('to_s'))) {
+            $sub->($_[0]);
+        }
+        else {
+            $_[0];
+        }
+      },
+      q{cmp} => sub {
         my ($obj1, $obj2, $first) = @_;
 
         if (CORE::ref($obj1) && $obj1->SUPER::isa(CORE::ref($obj2)) or CORE::ref($obj2) && $obj2->SUPER::isa(CORE::ref($obj1)))
         {
             if (defined(my $sub = $obj1->can('<=>'))) {
+                my $result = $sub->($obj1, $obj2);
                 local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                return $sub->($obj1, $obj2)->get_value;
+                return $result->get_value;
             }
         }
 
@@ -39,13 +61,13 @@ package Sidef::Object::Object {
     }
 
     sub say {
-        Sidef::Types::Bool::Bool->new(say @_);
+        Sidef::Types::Bool::Bool->new(CORE::say @_);
     }
 
     *println = \&say;
 
     sub print {
-        Sidef::Types::Bool::Bool->new(print @_);
+        Sidef::Types::Bool::Bool->new(CORE::print @_);
     }
 
     sub method {
