@@ -120,19 +120,19 @@ package Sidef::Sys::Sys {
 
     sub print {
         my ($self, @args) = @_;
-        Sidef::Types::Bool::Bool->new(print @args);
+        Sidef::Types::Bool::Bool->new(CORE::print(@args));
     }
 
     sub printf {
         my ($self, @args) = @_;
-        Sidef::Types::Bool::Bool->new(printf @args);
+        Sidef::Types::Bool::Bool->new(CORE::printf(@args));
     }
 
     sub printh {
         my ($self, $fh, @args) = @_;
 
         if (CORE::ref($fh) eq 'GLOB') {
-            return Sidef::Types::Bool::Bool->new(print {$fh} @args);
+            return Sidef::Types::Bool::Bool->new(CORE::print {$fh} @args);
         }
 
         $fh->print(@args);
