@@ -228,7 +228,7 @@ package Sidef::Types::Array::Array {
     sub concat {
         my ($self, $arg) = @_;
 
-        defined($arg) && $arg->isa('ARRAY')
+        eval { $arg->isa('ARRAY') }
           ? $self->new(@{$self}, @{$arg})
           : $self->new(@{$self}, $arg);
     }
@@ -285,7 +285,7 @@ package Sidef::Types::Array::Array {
         my ($self, $obj) = @_;
 
         my $counter = 0;
-        if (ref($obj) eq 'Sidef::Types::Block::Code') {
+        if (ref($obj) eq 'Sidef::Types::Block::Block') {
 
             foreach my $item (@{$self}) {
                 if ($obj->run($item)) {
@@ -443,17 +443,6 @@ package Sidef::Types::Array::Array {
         $_[0]->_min_max_by($_[1], -1);
     }
 
-    sub last {
-        my ($self, $arg) = @_;
-
-        if (defined $arg) {
-            my $from = @{$self} - $arg->get_value;
-            return $self->new(@{$self}[($from < 0 ? 0 : $from) .. $#{$self}]);
-        }
-
-        @{$self} ? $self->[-1] : ();
-    }
-
     sub swap {
         my ($self, $i, $j) = @_;
         @{$self}[$i, $j] = @{$self}[$j, $i];
@@ -470,16 +459,35 @@ package Sidef::Types::Array::Array {
         my ($self, $arg) = @_;
 
         if (defined $arg) {
-            if (ref($arg) eq 'Sidef::Types::Block::Code') {
-                return return $self->find($arg);
+
+            if (ref($arg) eq 'Sidef::Types::Block::Block') {
+                return $self->first_by($arg);
             }
 
-            return $self->new(@{$self}[0 .. $arg->get_value - 1]);
+            my $max = $#{$self};
+            $arg = $arg->get_value - 1;
+            return $self->new(@{$self}[0 .. ($arg > $max ? $max : $arg)]);
         }
 
         @{$self} ? $self->[0] : ();
     }
 
+    sub last {
+        my ($self, $arg) = @_;
+
+        if (defined $arg) {
+
+            if (ref($arg) eq 'Sidef::Types::Block::Block') {
+                return $self->last_by($arg);
+            }
+
+            my $from = @{$self} - $arg->get_value;
+            return $self->new(@{$self}[($from < 0 ? 0 : $from) .. $#{$self}]);
+        }
+
+        @{$self} ? $self->[-1] : ();
+    }
+
     sub _flatten {    # this exists for performance reasons
         my ($self) = @_;
 
@@ -531,6 +539,18 @@ package Sidef::Types::Array::Array {
         exists($self->[$index]) ? $self->[$index] : $default;
     }
 
+    sub dig {
+        my ($self, $key, @keys) = @_;
+
+        my $value = $self->fetch($key) // return;
+
+        foreach my $key (@keys) {
+            $value = $value->fetch($key) // return;
+        }
+
+        $value;
+    }
+
     sub _slice {
         my ($self, $from, $to) = @_;
 
@@ -658,7 +678,24 @@ package Sidef::Types::Array::Array {
         $hash;
     }
 
-    sub find {
+    sub freq {
+        my ($self) = @_;
+
+        my %hash;
+        foreach my $item (@{$self}) {
+            $hash{$item}++;
+        }
+
+        foreach my $key (keys %hash) {
+            $hash{$key} = Sidef::Types::Number::Number->new($hash{$key});
+        }
+
+        Sidef::Types::Hash::Hash->new(%hash);
+    }
+
+    *frequency = \&freq;
+
+    sub first_by {
         my ($self, $code) = @_;
         foreach my $val (@{$self}) {
             return $val if $code->run($val);
@@ -666,7 +703,15 @@ package Sidef::Types::Array::Array {
         return;
     }
 
-    *first_by = \&find;
+    *find = \&first_by;
+
+    sub last_by {
+        my ($self, $code) = @_;
+        for (my $i = $#{$self} ; $i >= 0 ; --$i) {
+            return $self->[$i] if $code->run($self->[$i]);
+        }
+        return;
+    }
 
     sub any {
         my ($self, $code) = @_;
@@ -711,9 +756,9 @@ package Sidef::Types::Array::Array {
     sub index {
         my ($self, $obj) = @_;
 
-        if (defined $obj) {
+        if (@_ > 1) {
 
-            if (ref($obj) eq 'Sidef::Types::Block::Code') {
+            if (ref($obj) eq 'Sidef::Types::Block::Block') {
                 foreach my $i (0 .. $#{$self}) {
                     $obj->run($self->[$i])
                       && return Sidef::Types::Number::Number->new($i);
@@ -737,8 +782,8 @@ package Sidef::Types::Array::Array {
     sub rindex {
         my ($self, $obj) = @_;
 
-        if (defined $obj) {
-            if (ref($obj) eq 'Sidef::Types::Block::Code') {
+        if (@_ > 1) {
+            if (ref($obj) eq 'Sidef::Types::Block::Block') {
                 for (my $i = $#{$self} ; $i >= 0 ; $i--) {
                     $obj->run($self->[$i])
                       && return Sidef::Types::Number::Number->new($i);
@@ -768,7 +813,7 @@ package Sidef::Types::Array::Array {
           && return $self->new;
 
         my @array;
-        if (ref($obj) eq 'Sidef::Types::Block::Code') {
+        if (ref($obj) eq 'Sidef::Types::Block::Block') {
             for (my $i = 1 ; $i <= $end ; $i += 2) {
                 push @array, scalar $obj->run($self->[$i - 1], $self->[$i]);
             }
@@ -819,7 +864,7 @@ package Sidef::Types::Array::Array {
     sub reduce {
         my ($self, $obj) = @_;
 
-        if (ref($obj) eq 'Sidef::Types::Block::Code') {
+        if (ref($obj) eq 'Sidef::Types::Block::Block') {
             (my $end = $#{$self}) >= 0 || return;
 
             my $x = $self->[0];
@@ -988,8 +1033,8 @@ package Sidef::Types::Array::Array {
     sub abbrev {
         my ($self, $code) = @_;
 
-        my $__END__ = {};                                                               # some unique value
-        my $__CALL__ = defined($code) && (ref($code) eq 'Sidef::Types::Block::Code');
+        my $__END__ = {};                                                                # some unique value
+        my $__CALL__ = defined($code) && (ref($code) eq 'Sidef::Types::Block::Block');
 
         my %table;
         foreach my $sub_array (@{$self}) {
@@ -1046,7 +1091,7 @@ package Sidef::Types::Array::Array {
     sub contains {
         my ($self, $obj) = @_;
 
-        if (ref($obj) eq 'Sidef::Types::Block::Code') {
+        if (ref($obj) eq 'Sidef::Types::Block::Block') {
             foreach my $item (@{$self}) {
                 if ($obj->run($item)) {
                     return Sidef::Types::Bool::Bool->true;
@@ -1065,7 +1110,9 @@ package Sidef::Types::Array::Array {
         Sidef::Types::Bool::Bool->false;
     }
 
-    *contain = \&contains;
+    *contain  = \&contains;
+    *include  = \&contains;
+    *includes = \&contains;
 
     sub contains_type {
         my ($self, $obj) = @_;
@@ -1141,7 +1188,8 @@ package Sidef::Types::Array::Array {
         CORE::splice(@{$self}, $offset->get_value, 1);
     }
 
-    *pop_at = \&delete_index;
+    *pop_at    = \&delete_index;
+    *delete_at = \&delete_index;
 
     sub splice {
         my ($self, $offset, $length, @objects) = @_;
@@ -1285,7 +1333,7 @@ package Sidef::Types::Array::Array {
         my ($self, $num) = @_;
 
         $num = $num->get_value % ($#{$self} + 1);
-        return $self if $num == 0;
+        return $self->new(@{$self}) if $num == 0;
 
         # Surprisingly, this is slower:
         # $self->new(@{$self}[$num .. $#{$self}], @{$self}[0 .. $num - 1]);
