@@ -30,6 +30,8 @@ package Sidef::Types::Glob::FileHandle {
         Sidef::Types::Bool::Bool->new(-t $_[0]{fh});
     }
 
+    *isatty = \&is_on_tty;
+
     sub stdout {
         __PACKAGE__->new(fh => \*STDOUT);
     }
@@ -58,8 +60,6 @@ package Sidef::Types::Glob::FileHandle {
         Sidef::Types::Bool::Bool->new(CORE::syswrite $self->{fh}, @args);
     }
 
-    *write_string = \&syswrite;
-
     sub print {
         my ($self, @args) = @_;
         Sidef::Types::Bool::Bool->new(CORE::print {$self->{fh}} @args);
