@@ -8,7 +8,7 @@ package Sidef::Types::Hash::Hash {
       );
 
     use overload
-      q{bool} => sub { scalar(keys %{$_[0]}) },
+      q{bool} => sub { scalar(CORE::keys %{$_[0]}) },
       q{""}   => \&dump;
 
     sub new {
@@ -22,7 +22,7 @@ package Sidef::Types::Hash::Hash {
         my ($self) = @_;
 
         my %hash;
-        foreach my $k (keys %{$self}) {
+        foreach my $k (CORE::keys %$self) {
             my $v = $self->{$k};
             $hash{$k} = (
                          index(ref($v), 'Sidef::') == 0
@@ -49,6 +49,18 @@ package Sidef::Types::Hash::Hash {
         exists($self->{$key}) ? $self->{$key} : $default;
     }
 
+    sub dig {
+        my ($self, $key, @keys) = @_;
+
+        my $value = $self->fetch($key) // return;
+
+        foreach my $key (@keys) {
+            $value = $value->fetch($key) // return;
+        }
+
+        $value;
+    }
+
     sub slice {
         my ($self, @keys) = @_;
         $self->new(map { ($_ => exists($self->{$_}) ? $self->{$_} : undef) } @keys);
@@ -56,7 +68,7 @@ package Sidef::Types::Hash::Hash {
 
     sub length {
         my ($self) = @_;
-        Sidef::Types::Number::Number->new(scalar CORE::keys %{$self});
+        Sidef::Types::Number::Number->new(scalar CORE::keys %$self);
     }
 
     *len = \&length;
@@ -64,10 +76,10 @@ package Sidef::Types::Hash::Hash {
     sub eq {
         my ($self, $obj) = @_;
 
-        (%{$self} eq %{$obj})
+        (%$self eq %{$obj})
           or return Sidef::Types::Bool::Bool->false;
 
-        while (my ($key, $value) = each %{$self}) {
+        while (my ($key, $value) = each %$self) {
             exists($obj->{$key})
               or return Sidef::Types::Bool::Bool->false;
 
@@ -87,11 +99,11 @@ package Sidef::Types::Hash::Hash {
         my ($self, $obj) = @_;
 
         if (ref($self) ne ref($obj)
-            or %{$self} ne %{$obj}) {
+            or %$self ne %{$obj}) {
             return Sidef::Types::Bool::Bool->false;
         }
 
-        while (my ($key) = each %{$self}) {
+        while (my ($key) = each %$self) {
             exists($obj->{$key})
               or return Sidef::Types::Bool::Bool->false;
         }
@@ -102,7 +114,7 @@ package Sidef::Types::Hash::Hash {
     sub append {
         my ($self, %pairs) = @_;
 
-        foreach my $key (keys %pairs) {
+        foreach my $key (CORE::keys %pairs) {
             $self->{$key} = $pairs{$key};
         }
 
@@ -118,63 +130,79 @@ package Sidef::Types::Hash::Hash {
           : (delete @{$self}{@keys});
     }
 
-    sub _iterate {
-        my ($self, $code, $callback) = @_;
-
-        while (my ($key, $value) = each %{$self}) {
-            my $key_obj = Sidef::Types::String::String->new($key);
-
-            if ($code->run($key_obj, $value)) {
-                $callback->($key, $value);
-            }
-        }
-
-        $self;
-    }
-
     sub map_val {
         my ($self, $code) = @_;
 
-        while (my ($key, $value) = each %{$self}) {
-            $self->{$key} = $code->run(Sidef::Types::String::String->new($key), $value);
+        my %hash;
+        foreach my $key (CORE::keys %$self) {
+            $hash{$key} = $code->run(Sidef::Types::String::String->new($key), $self->{$key});
         }
 
-        $self;
+        $self->new(%hash);
+    }
+
+    *map_values = \&map_val;
+
+    sub map {
+        my ($self, $code) = @_;
+
+        my %hash;
+        foreach my $key (CORE::keys %$self) {
+            my ($k, $v) = $code->run(Sidef::Types::String::String->new($key), $self->{$key});
+            $hash{$k} = $v;
+        }
+
+        $self->new(%hash);
     }
 
     sub select {
         my ($self, $code) = @_;
 
-        my @pairs;
-        $self->_iterate(
-            $code,
-            sub {
-                push @pairs, @_;
+        my %hash;
+        foreach my $key (CORE::keys %$self) {
+            my $value = $self->{$key};
+            if ($code->run(Sidef::Types::String::String->new($key), $value)) {
+                $hash{$key} = $value;
             }
-        );
+        }
 
-        $self->new(@pairs);
+        $self->new(%hash);
     }
 
     *grep = \&select;
 
-    sub delete_if {
+    sub count {
         my ($self, $code) = @_;
-        $self->_iterate(
-            $code,
-            sub {
-                delete $self->{$_[0]};
+
+        my $count = 0;
+        foreach my $key (CORE::keys %$self) {
+            if ($code->run(Sidef::Types::String::String->new($key), $self->{$key})) {
+                ++$count;
             }
-        );
+        }
+
+        Sidef::Types::Number::Number->new($count);
     }
 
-    *deleteIf = \&delete_if;
+    *count_by = \&count;
+
+    sub delete_if {
+        my ($self, $code) = @_;
+
+        foreach my $key (CORE::keys %$self) {
+            if ($code->run(Sidef::Types::String::String->new($key), $self->{$key})) {
+                delete($self->{$key});
+            }
+        }
+
+        $self;
+    }
 
     sub concat {
         my ($self, $obj) = @_;
 
         my @list;
-        while (my ($key, $val) = each %{$self}) {
+        while (my ($key, $val) = each %$self) {
             push @list, $key, $val;
         }
 
@@ -190,7 +218,7 @@ package Sidef::Types::Hash::Hash {
     sub merge_values {
         my ($self, $obj) = @_;
 
-        while (my ($key, undef) = each %{$self}) {
+        while (my ($key, undef) = each %$self) {
             if (exists $obj->{$key}) {
                 $self->{$key} = $obj->{$key};
             }
@@ -201,18 +229,18 @@ package Sidef::Types::Hash::Hash {
 
     sub keys {
         my ($self) = @_;
-        Sidef::Types::Array::Array->new(map { Sidef::Types::String::String->new($_) } keys %{$self});
+        Sidef::Types::Array::Array->new(map { Sidef::Types::String::String->new($_) } CORE::keys %$self);
     }
 
     sub values {
         my ($self) = @_;
-        Sidef::Types::Array::Array->new(values %{$self});
+        Sidef::Types::Array::Array->new(CORE::values %$self);
     }
 
     sub each_value {
         my ($self, $code) = @_;
 
-        foreach my $value (CORE::values %{$self}) {
+        foreach my $value (CORE::values %$self) {
             if (defined(my $res = $code->_run_code($value))) {
                 return $res;
             }
@@ -224,7 +252,7 @@ package Sidef::Types::Hash::Hash {
     sub each_key {
         my ($self, $code) = @_;
 
-        foreach my $key (CORE::keys %{$self}) {
+        foreach my $key (CORE::keys %$self) {
             if (defined(my $res = $code->_run_code(Sidef::Types::String::String->new($key)))) {
                 return $res;
             }
@@ -238,7 +266,7 @@ package Sidef::Types::Hash::Hash {
 
         if (defined($obj)) {
 
-            foreach my $key (CORE::keys %{$self}) {
+            foreach my $key (CORE::keys %$self) {
                 if (defined(my $res = $obj->_run_code(Sidef::Types::String::String->new($key), $self->{$key}))) {
                     return $res;
                 }
@@ -247,7 +275,7 @@ package Sidef::Types::Hash::Hash {
             return $obj;
         }
 
-        my ($key, $value) = each(%{$self});
+        my ($key, $value) = each(%$self);
 
         $key // return;
         Sidef::Types::Array::Array->new(Sidef::Types::String::String->new($key), $value);
@@ -259,8 +287,8 @@ package Sidef::Types::Hash::Hash {
         my ($self, $code) = @_;
 
         my @array;
-        while (my ($key, $value) = CORE::each %{$self}) {
-            push @array, [$key, $code->run(Sidef::Types::String::String->new($key), $value)];
+        foreach my $key (CORE::keys %$self) {
+            push @array, [$key, $code->run(Sidef::Types::String::String->new($key), $self->{$key})];
         }
 
         Sidef::Types::Array::Array->new(
@@ -275,7 +303,7 @@ package Sidef::Types::Hash::Hash {
         Sidef::Types::Array::Array->new(
             map {
                 Sidef::Types::Array::Pair->new(Sidef::Types::String::String->new($_), $self->{$_})
-              } CORE::keys %{$self}
+              } CORE::keys %$self
         );
     }
 
@@ -288,14 +316,17 @@ package Sidef::Types::Hash::Hash {
     }
 
     *has_key  = \&exists;
+    *contain  = \&exists;
     *contains = \&exists;
+    *include  = \&exists;
+    *includes = \&exists;
 
     sub flip {
         my ($self) = @_;
 
         my $new_hash = $self->new();
-        @{$new_hash}{map { $_->get_value } CORE::values %{$self}} =
-          (map           { Sidef::Types::String::String->new($_) } CORE::keys %{$self});
+        @{$new_hash}{map { $_->get_value } CORE::values %$self} =
+          (map           { Sidef::Types::String::String->new($_) } CORE::keys %$self);
 
         $new_hash;
     }
@@ -313,7 +344,7 @@ package Sidef::Types::Hash::Hash {
         $Sidef::SPACES += $Sidef::SPACES_INCR;
 
         # Sort the keys case insensitively
-        my @keys = sort { (lc($a) cmp lc($b)) || ($a cmp $b) } CORE::keys(%{$self});
+        my @keys = sort { (lc($a) cmp lc($b)) || ($a cmp $b) } CORE::keys(%$self);
 
         my $str = Sidef::Types::String::String->new(
             "Hash(" . (
