@@ -93,13 +93,13 @@ package Sidef::Types::Number::Complex {
 
     sub cartesian {
         my ($self) = @_;
-        ${$self}->display_format('cartesian');
+        $$self->display_format('cartesian');
         $self;
     }
 
     sub polar {
         my ($self) = @_;
-        ${$self}->display_format('polar');
+        $$self->display_format('polar');
         $self;
     }
 
@@ -110,6 +110,20 @@ package Sidef::Types::Number::Complex {
 
     *re = \&real;
 
+    sub parts {
+        my ($self) = @_;
+        map { Sidef::Types::Number::Number->new($_) } @{$$self->_cartesian};
+    }
+
+    *reals = \&parts;
+
+    sub polars {
+        my ($self) = @_;
+        map { Sidef::Types::Number::Number->new($_) } @{$$self->_polar};
+    }
+
+    *polar_parts = \&polars;
+
     sub imaginary {
         my ($self) = @_;
         $self->new(Math::Complex::Im($$self));
@@ -250,6 +264,18 @@ package Sidef::Types::Number::Complex {
         $self->new($$self**$arg->get_value);
     }
 
+    sub root {
+        my ($self, $n, $k) = @_;
+        local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+        $self->new($$self->root($n->get_value, $k->get_value));
+    }
+
+    sub roots {
+        my ($self, $n) = @_;
+        local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+        Sidef::Types::Array::Array->new(map { $self->new($_) } $$self->root($n->get_value));
+    }
+
     sub int {
         my ($self) = @_;
         $self->new(CORE::int($$self));
