@@ -201,14 +201,6 @@ package Sidef::Types::String::String {
         $self->times($num);
     }
 
-    sub uc {
-        my ($self) = @_;
-        $self->new(CORE::uc $$self);
-    }
-
-    *upcase = \&uc;
-    *upper  = \&uc;
-
     sub equals {
         my ($self, $arg) = @_;
         Sidef::Types::Bool::Bool->new($$self eq $$arg);
@@ -250,6 +242,14 @@ package Sidef::Types::String::String {
     *downcase = \&lc;
     *lower    = \&lc;
 
+    sub uc {
+        my ($self) = @_;
+        $self->new(CORE::uc $$self);
+    }
+
+    *upcase = \&uc;
+    *upper  = \&uc;
+
     sub fc {
         my ($self) = @_;
         $self->new(CORE::fc $$self);
@@ -469,7 +469,7 @@ package Sidef::Types::String::String {
     sub sub {
         my ($self, $regex, $str) = @_;
 
-        ref($str) eq 'Sidef::Types::Block::Code'
+        ref($str) eq 'Sidef::Types::Block::Block'
           && return $self->esub($regex, $str);
 
         $str //= __PACKAGE__->new('');
@@ -485,7 +485,7 @@ package Sidef::Types::String::String {
     sub gsub {
         my ($self, $regex, $str) = @_;
 
-        ref($str) eq 'Sidef::Types::Block::Code'
+        ref($str) eq 'Sidef::Types::Block::Block'
           && return $self->gesub($regex, $str);
 
         $str //= __PACKAGE__->new('');
@@ -506,7 +506,7 @@ package Sidef::Types::String::String {
         $code //= __PACKAGE__->new('');
         my $search = $self->_string_or_regex($regex);
 
-        if (ref($code) eq 'Sidef::Types::Block::Code') {
+        if (ref($code) eq 'Sidef::Types::Block::Block') {
             return __PACKAGE__->new($$self =~ s{$search}{$code->run(_get_captures($$self))}er);
         }
 
@@ -519,7 +519,7 @@ package Sidef::Types::String::String {
         $code //= __PACKAGE__->new('');
         my $search = $self->_string_or_regex($regex);
 
-        if (ref($code) eq 'Sidef::Types::Block::Code') {
+        if (ref($code) eq 'Sidef::Types::Block::Block') {
             my $value = $$self;
             return __PACKAGE__->new($value =~ s{$search}{$code->run(_get_captures($value))}ger);
         }
@@ -592,14 +592,43 @@ package Sidef::Types::String::String {
         __PACKAGE__->new($acc);
     }
 
-    sub each_word {
-        my ($self, $obj) = @_;
-        my $array = Sidef::Types::Array::Array->new(map { __PACKAGE__->new($_) } CORE::split(' ', $$self));
-        $obj // return $array;
-        $array->each($obj);
+    sub words {
+        my ($self) = @_;
+        Sidef::Types::Array::Array->new(map { __PACKAGE__->new($_) } CORE::split(' ', $$self));
     }
 
-    *words = \&each_word;
+    sub each_word {
+        my ($self, $code) = @_;
+
+        foreach my $word (CORE::split(' ', $$self)) {
+            if (defined(my $res = $code->_run_code(__PACKAGE__->new($word)))) {
+                return $res;
+            }
+        }
+
+        $self;
+    }
+
+    sub numbers {
+        my ($self) = @_;
+        Sidef::Types::Array::Array->new(map { Sidef::Types::Number::Number->new($_) } CORE::split(' ', $$self));
+    }
+
+    *nums = \&numbers;
+
+    sub each_number {
+        my ($self, $code) = @_;
+
+        foreach my $num (CORE::split(' ', $$self)) {
+            if (defined(my $res = $code->_run_code(Sidef::Types::Number::Number->new($num)))) {
+                return $res;
+            }
+        }
+
+        $self;
+    }
+
+    *each_num = \&each_number;
 
     sub bytes {
         my ($self) = @_;
@@ -908,7 +937,9 @@ package Sidef::Types::String::String {
         Sidef::Types::Bool::Bool->new(CORE::index($$self, $arg->get_value, $start_pos) != -1);
     }
 
-    *include = \&contains;
+    *contain  = \&contains;
+    *include  = \&contains;
+    *includes = \&contains;
 
     sub count {
         my ($self, $arg) = @_;
@@ -921,7 +952,7 @@ package Sidef::Types::String::String {
             $counter += CORE::length($1) while $s =~ /($regex)/g;
             return Sidef::Types::Number::Number->new($counter);
         }
-        elsif (ref($arg) eq 'Sidef::Types::Block::Code') {
+        elsif (ref($arg) eq 'Sidef::Types::Block::Block') {
             foreach my $char (split //, $s) {
                 ++$counter if $arg->run(__PACKAGE__->new($char));
             }
